@@ -11,6 +11,9 @@ FILE_TYPE_KEY = "FILE"
 STRING_ARRAY_TYPE_KEY = "STRING_ARRAY"
 FILE_ARRAY_TYPE_KEY = "FILE_ARRAY"
 
+# file upload size limit
+MAX_FILE_UPLOAD_SIZE_BYTES = 50 * 1025 * 1024 * 1024  # 50GB
+
 # support text
 SUPPORT_EMAIL = "scientific-services-support@broadinstitute.org"
 SUPPORT_EMAIL_TEXT = f"For help troubleshooting, email {SUPPORT_EMAIL}"
